/*
 * Copyright (c) 2005 Your Corporation. All Rights Reserved.
 */
package org.activemq.transport.stomp;

import org.activemq.message.ActiveMQDestination;
import org.activemq.message.ActiveMQTextMessage;

import javax.jms.JMSException;
import java.io.ByteArrayOutputStream;
import java.io.DataInput;
import java.io.IOException;
import java.net.ProtocolException;
import java.util.Properties;

class Send implements Command
{
    private final HeaderParser parser = new HeaderParser();
    private final StompWireFormat format;

    Send(StompWireFormat format)
    {
        this.format = format;
    }

    public PacketEnvelope build(String commandLine, DataInput in) throws IOException
    {
        Properties headers = parser.parse(in);
        String destination = (String) headers.remove(Stomp.Headers.Send.DESTINATION);

        ActiveMQTextMessage text = new ActiveMQTextMessage();
        text.setJMSMessageID(StompWireFormat.PACKET_IDS.generateId());

        ActiveMQDestination d = DestinationNamer.convert(destination);
        text.setJMSDestination(d);
        text.setJMSClientID(format.getClientId());

        // the standard JMS headers
        text.setJMSCorrelationID((String) headers.remove(Stomp.Headers.Send.CORRELATION_ID));

        Object expiration = headers.remove(Stomp.Headers.Send.EXPIRATION_TIME);
        if (expiration != null)
        {
            text.setJMSExpiration(asLong(expiration));
        }
        Object priority = headers.remove(Stomp.Headers.Send.PRORITY);
        if (priority != null)
        {
            text.setJMSExpiration(asInt(priority));
        }

        text.setJMSReplyTo(DestinationNamer.convert((String) headers.remove(Stomp.Headers.Send.REPLY_TO)));

        // now the general headers
        text.setProperties(headers);

        // now the body
        ByteArrayOutputStream bytes = new ByteArrayOutputStream();
        byte b;
        while ((b = in.readByte()) != 0)
        {
            bytes.write(b);
        }
        bytes.close();
        String body = new String(bytes.toByteArray());
        try
        {
            text.setText(body);
        }
        catch (JMSException e)
        {
            throw new RuntimeException("Something is really wrong, we instantiated this thing!");
        }

        if (headers.containsKey(Stomp.Headers.TRANSACTION))
        {
            String tx_id = format.getTransactionId(headers.getProperty(Stomp.Headers.TRANSACTION));
            if (tx_id == null) throw new ProtocolException(headers.getProperty(Stomp.Headers.TRANSACTION) +
                                                           " is an invalid transaction id");
            text.setTransactionIDString(tx_id);
        }
        else if (format.isInDefaultTransaction())
        {
            String tx_id = format.getTransactionId(Stomp.Headers.DEFAULT_TRANSACTION_NAME);
            text.setTransactionIDString(tx_id);
        }

        return new PacketEnvelope(text, headers);
    }

    protected boolean asBool(Object value)
    {
        if (value != null)
        {
            return value.toString().equals("true");
        }
        return false;
    }

    protected long asLong(Object value)
    {
        if (value instanceof Number)
        {
            Number n = (Number) value;
            return n.longValue();
        }
        return Long.parseLong(value.toString());
    }

    protected int asInt(Object value)
    {
        if (value instanceof Number)
        {
            Number n = (Number) value;
            return n.intValue();
        }
        return Integer.parseInt(value.toString());
    }
}
