/**
 * 
 * Copyright 2005 LogicBlaze, Inc. http://www.logicblaze.com
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/
package org.activemq.bug;

import org.activemq.test.EmbeddedBrokerTestSupport;

import javax.jms.Connection;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageListener;
import javax.jms.Session;

/**
 * Test case for AMQ-317
 * 
 * @author Jason Anderson
 * @version $Revision: 1.1 $
 */
public class AMQ317Test extends EmbeddedBrokerTestSupport {

    protected long[] received = new long[1];
    private long group = 10;
    protected long publishSleepTime = 0;
    protected boolean verbose;

    public void testAMQ317() throws Exception {
        Connection connection = connectionFactory.createConnection();
        Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
        MessageConsumer consumer = session.createConsumer(destination);
        consumer.setMessageListener(new MessageListener() {
            public void onMessage(Message message) {
                if (verbose) {
                    System.out.println("Received: " + message);
                }

                synchronized (received) {
                    received[0] += 1;
                }

                if (verbose) {
                    System.out.println("Now value is: " + received[0]);
                }
            }
        });
        connection.start();

        final int total = 1000000;

        new Thread(new Runnable() {
            public void run() {
                for (int i = 0; i < total; i++) {
                    template.convertAndSend(getDestinationString(), "test message");
                    if (publishSleepTime > 0) {
                        try {
                            Thread.sleep(publishSleepTime);
                        }
                        catch (InterruptedException e) {
                            System.out.println("Caught: " + e);
                        }
                    }
                }
            }
        }).start();

        long last = 0;
        long cur = 0;

        do {
            synchronized (received) {
                cur = received[0];
            }
            if (cur % group == 0) {
                System.out.println("received = " + cur + " diff = " + (cur - last));
                last = cur;
            }
            Thread.sleep(1000);
        }
        while (cur < total);
    }
}
