import org.activemq.openwire.tool.OpenWireScript

/**
 * Generates the Ruby marshalling code for the Open Wire Format
 *
 * @version $Revision: 1.1.1.1 $
 */
class GenerateRubyMarshalling extends OpenWireScript {

    Object run() {
        def destDir = new File("language/ruby/src/org/activemq/openwire")
        destDir.mkdirs()

        def messageClasses = classes.findAll { isMessageType(it) }

        println "Generating Ruby marshalling code to directory ${destDir}"

        def buffer = new StringBuffer()

        int counter = 0
        Map map = [:]

        for (jclass in messageClasses) {

            println "Processing $jclass.simpleName"

            def properties = jclass.declaredProperties.findAll { isValidProperty(it) }
            def file = new File(destDir, jclass.simpleName + "Marshaller.rb")

            String baseClass = "AbstractPacketMarshaller"
            if (jclass.superclass?.simpleName == "ActiveMQMessage") {
                baseClass = "ActiveMQMessageMarshaller"
            }

            buffer << """
${jclass.simpleName}Marshaller.class

"""

            file.withWriter { out |
                out << """#
# Marshalling code for Open Wire Format for ${jclass.simpleName}
#
#
# NOTE!: This file is autogenerated - do not modify!
#        if you need to make a change, please see the Groovy scripts in the
#        activemq-openwire module
#

class ${jclass.simpleName} < $baseClass

  def packet_type()
    return ${getEnum(jclass)};
  end

  def create_packet()
    return ${jclass.simpleName}.new;
  end

  def build_packet(packet, data_in)
    super.build_packet(packet, data_in)
"""
for (property in properties) {
  out << """
    packet.${property.setter.simpleName}("""

                    def type = property.type.qualifiedName
                    if (isPropertiesType(type)) {
                        out << "readProperties(data_in)"
                    }
                    else if (isDestinationType(type)) {
                        out << "readDestination(data_in)"
                    }
                    else {
                        switch (type) {
                            case "java.lang.String":
                                out << "data_in.gets"
                                break;

                            case "boolean":
                                out << "data_in.readBoolean()"
                                break;

                            case "byte":
                                out << "data_in.readByte()"
                                break;

                            case "byte[]":
                                out << "readBytes(data_in)"
                                break;

                            case "char":
                                out << "data_in.readChar()"
                                break;

                            case "short":
                                out << "dataIn.readShort()"
                                break;

                            case "int":
                                out << "dataIn.readInt()"
                                break;

                            case "long":
                                out << "dataIn.readLong()"
                                break;

                            case "float":
                                out << "dataIn.readFloat()"
                                break;

                            case "double":
                                out << "data_in.readDouble()"
                                break;

                            default:
                                out << "(${type}) readObject(data_in)"
                        }
                    }
                    out << """);
"""
                }

                out << """
  end

  #public override void WritePacket(Packet packet, DataOutput dataOut) throws IOException {
  def write_packet(packet, data_out)
    super.writePacket(packet, data_out);

"""
                for (property in properties) {
                    def getter = "info." + property.getter.simpleName + "()"
                    out << "    "

                    def type = property.type.qualifiedName
                    if (isPropertiesType(type)) {
                        out << "writeProperties($getter, data_out);"
                    }
                    else if (isDestinationType(type)) {
                        out << "writeDestination($getter, data_out);"
                    }
                    else {
                        switch (type) {
                            case "java.lang.String":
                                out << "data_out.puts($getter);"
                                break;

                            case "boolean":
                                out << "data_out.writeBoolean($getter);"
                                break;

                            case "byte":
                                out << "data_out.writeByte($getter);"
                                break;

                            case "byte[]":
                                out << "writeBytes($getter, data_out);"
                                break;

                            case "char":
                                out << "data_out.writeChar($getter);"
                                break;

                            case "short":
                                out << "data_out.writeShort($getter);"
                                break;

                            case "int":
                                out << "data_out.writeInt($getter);"
                                break;

                            case "long":
                                out << "data_out.writeLong($getter);"
                                break;

                            case "float":
                                out << "data_out.writeFloat($getter);"
                                break;

                            case "double":
                                out << "data_out.writeDouble($getter);"
                                break;

                            default:
                                out << "writeObject($getter, data_out);"
                        }
                    }
                    out << """
"""
                }

                out << """
    end
  end
end
"""
            }
        }
    }
}