/**
 * 
 * Copyright 2005 LogicBlaze, Inc. http://www.logicblaze.com
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/
package org.activemq.systest.impl;

import org.activemq.ActiveMQConnectionFactory;
import org.activemq.broker.BrokerContainer;
import org.activemq.broker.impl.BrokerContainerImpl;
import org.activemq.systest.AgentStopper;
import org.activemq.systest.AgentSupport;
import org.activemq.systest.BrokerAgent;

import javax.jms.ConnectionFactory;

/**
 * 
 * @version $Revision: 1.1 $
 */
public class OlderBrokerAgentImpl extends AgentSupport implements BrokerAgent {

    private static int port = 61616;

    private String connectionURI;
    private BrokerContainer broker;
    private boolean peristent;

    
    public OlderBrokerAgentImpl() {
        connectionURI = "tcp://localhost:" + (port++);
    }

    public void kill() throws Exception {
        stop();
    }

    public ConnectionFactory getConnectionFactory() {
        return new ActiveMQConnectionFactory(connectionURI);
    }

    public void connectTo(BrokerAgent remoteBroker) throws Exception {
        getBroker().addNetworkConnector(remoteBroker.getConnectionURI());
    }

    public void start() throws Exception {
        getBroker().start();
    }

    public void stop(AgentStopper stopper) {
        if (broker != null) {
            try {
                broker.stop();
            }
            catch (Exception e) {
                stopper.onException(this, e);
            }
            finally {
                broker = null;
            }
        }
    }

    public void setBroker(BrokerContainer broker) {
        this.broker = broker;
    }

    public String getConnectionURI() {
        return connectionURI;
    }

    public void setConnectionURI(String brokerURL) {
        this.connectionURI = brokerURL;
    }

    public boolean isPeristent() {
        return peristent;
    }

    public void setPeristent(boolean peristent) {
        this.peristent = peristent;
    }

    public BrokerContainer getBroker() throws Exception {
        if (broker == null) {
            broker = createBroker();
        }
        return broker;
    }

    protected BrokerContainer createBroker() throws Exception {
        BrokerContainerImpl answer = new BrokerContainerImpl();
        answer.addConnector(getConnectionURI());
        return answer;
    }
}
